/*
	File: HammWM.C
	Component of the Stochastic Discrimination Utilities
	Copyright (C) 2002, David C. Lambert <dcl@panix.com>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful, 
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software Foundation, Inc.,
	59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#include <math.h>
#include <string.h>
#include "HammWM.H"

double		HammWM::RhoMin;
double		HammWM::RhoMax;
double		HammWM::RhoDecay;

WeakModel *
HammWMFactory(WMSet &set, int nDims)
{
	return (new HammWM(nDims));
}

WeakModel *
HammWMCopier(const WeakModel &rhs)
{
	return (new HammWM((HammWM &)rhs));
}

WeakModel *
HammWMLoader(int nDims, FILE *dumpFile)
{
	return (new HammWM(nDims, dumpFile));
}

void
HammWMParamUpdate()
{
	if (HammWM::RhoMax > HammWM::RhoMin) HammWM::RhoMax *= HammWM::RhoDecay;
}

HammWM::HammWM(int nDims)
		: WeakModel(nDims), _distance(1), _prototype(new int [_setDims])
{
	double	*point0 = WMSet::RandomPositiveDataPoint();
	double	*point1 = WMSet::RandomNegativeDataPoint();

	_distance = 0;
	if (_setDims < _nDims) {
		for(int i = 0; i < _setDims; i++) {
			int		j = _inSet[i];
			_prototype[i] = (int)point0[j];
			if (point0[j] == point1[j])
				_distance++;
		}
	}
	else {
		for(int i = 0; i < _nDims; i++) {
			_prototype[i] = (int)point0[i];
			if (point0[i] == point1[i])
				_distance++;
		}
	}
	_distance--;
}

HammWM::HammWM(const HammWM &rhs)
		: WeakModel(rhs), _distance(rhs._distance), _prototype(new int [rhs._setDims])
{
	memcpy(_prototype, rhs._prototype, sizeof(int)*_setDims);
}

HammWM::HammWM(int nDims, FILE *dumpFile)
		: WeakModel(nDims, dumpFile), _distance(0), _prototype(new int [_setDims])
{
	fscanf(dumpFile, " { %d ", &_distance);
	for(int i = 0; i < _setDims; i++)
		fscanf(dumpFile, "%d ", &_prototype[i]);
	fscanf(dumpFile, "} ");
}

int
HammWM::containsPoint(double *p)
{
	int		distance = _setDims;

	if (_setDims < _nDims) {
		for(int i = 0; i < _setDims; i++) {
			int		j = _inSet[i];
			if (_prototype[i] == (int)p[j]) {
				distance--;
				if (distance < _distance)
					return 1;
			}
		}
	}
	else {
		for(int i = 0; i < _nDims; i++) {
			if (_prototype[i] == (int)p[i]) {
				distance--;
				if (distance < _distance)
					return 1;
			}
		}
	}
	return 0;
}

void
HammWM::writeToFile(FILE *dumpFile)
{
	WeakModel::writeToFile(dumpFile);
	fprintf(dumpFile, " { %d ", _distance);
	for(int i = 0; i < _setDims; i++)
		fprintf(dumpFile, "%d ", _prototype[i]);
	fprintf(dumpFile, "} ");
}

void
HammWM::print(FILE *printFile)
{
	fprintf(printFile, "%d %d", _distance, _setDims);
}
