/*
	File: SD.H
	Component of the Stochastic Discrimination Utilities
	Copyright (C) 2002, David C. Lambert <dcl@panix.com>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful, 
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software Foundation, Inc.,
	59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef _SD_H
#define _SD_H

#include <math.h>
#include <time.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include "WMSet.H"

class SD {
  public:
	SD(int nDims, int nData, int maxSets, double beta, double kappa, double tau, double rhoMin, double rhoMax, double rhoDecay,
		double **data, int *classes, DPair *bounds);

	SD(FILE *dumpFile);

	virtual ~SD() {
		delete [] _covered;
		delete [] _counters;
		delete [] _frequencies;
		delete [] _fracPos;
		delete [] _fracNeg;
		delete [] _posIndices;
		delete [] _negIndices;

		for(int i = 0; i< _nSets; i++) delete _sets[i];
		delete [] _sets;
	}

	int				nDims() const { return _nDims; }
	int				nData() const { return _nData; }
	int				nSets() const { return _nSets;}
	int				maxSets() const { return _maxSets; }
	double			nPositive() const { return _nPositive; }
	double			nNegative() const { return _nNegative; }
	double			tau() const { return _tau; }
	double			beta() const { return _beta; }
	double			kappa() const { return _kappa; }
	double			rhoMin() const { return _rhoMin; }
	double			rhoMax() const { return _rhoMax; }
	double			rhoDecay() const { return _rhoDecay; }
	const int		*posIndices() const { return _posIndices; }
	const int		*negIndices() const { return _negIndices; }

	//double			*randomPositivePoint() { return _data[_posIndices[int(drand48() * _nPositive)]]; }
	//double			*randomNegativePoint() { return _data[_negIndices[int(drand48() * _nPositive)]]; }

	void			writeToFile(FILE *dumpFile);

	void			getDistValues(double &minPos, double &maxPos, double &posAvg, double &posSD,
								  double &minNeg, double &maxNeg, double &negAvg, double &negSD);

	int				classifyPoint(double *p, double &avg, double thresh=0.500);

	void			calculateCoverages(WMSet &set);
	void			testUniformity(WMSet &set, double &avgPosNumber, double &avgNegNumber);

	void			takeNewSet(WMSet *newSet);
	void			updateVariables(WMSet &set);

	void			addSubset(int tries=-1);
	int				addSubset(WMSet &newWMSet, int &noEnrichment, int &noUniformity, int skipUniformity=0);

	double			getAccuracy(int dump=0);
	double			getAccuracy(int nData, double **data, int *classes, int dump=0, double thresh=0.500);

  protected:
	int				_nDims;
	int				_nData;
	int				_nSets;
	int				_maxSets;
	double			_nPositive;
	double			_nNegative;
	double			_tau;
	double			_beta;
	double			_kappa;
	double			_rhoMin;
	double			_rhoMax;
	double			_rhoDecay;
	double			_avgPosNumber;
	double			_avgNegNumber;

	int				*_classes;
	int				*_covered;
	int				*_posIndices;
	int				*_negIndices;

	double			*_counters;
	double			*_frequencies;
	double			*_fracPos;
	double			*_fracNeg;
	double			**_data;

	DPair			*_bounds;
	WMSet			**_sets;
};

#endif
