/*
    File: SDTest.C
    Component of the Stochastic Discrimination Utilities
    Copyright (C) 2002, David C. Lambert <dcl@panix.com>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful, 
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#include <string.h>
#include "SD.H"
#include "SDUtil.H"

void
usage()
{
	fprintf(stderr, "\n");
	fprintf(stderr, "Usage: SDTest -i nInputs -C nClasses -n nTest -f testFileName -o dumpFileName [-t thresh]\n");
	fprintf(stderr, "Defaults:\n");
	fprintf(stderr, "  thresh = 0.500\n");
	fprintf(stderr, "\n");
}

int
main(int argc, char **argv)
{
	int		c;
	int		i;
	int		j;
	int		k;
	int		nTest = 0;
	int		nInputs = 0;
	int		nClasses = 2;
	int		dispInterval = 10;
	int		modelType = (int)WMSet::HRECT;
	double	thresh = 0.500;
	char	testFileName[256];
	char	dumpFileName[256];
	extern char *optarg;
	extern int optind, opterr, optopt;

	strcpy(testFileName, "/dev/null");
	strcpy(dumpFileName, "/dev/null");

	while((c = getopt(argc, argv, "n:f:i:C:t:o:m:")) != -1) {
		switch(c) {
		  case 'n':
			  nTest = atoi(optarg);
			  break;
		  case 'm':
			  modelType = atoi(optarg);
			  break;
		  case 'f':
			  sprintf(testFileName, "%.255s", optarg);
			  break;
		  case 'o':
			  sprintf(dumpFileName, "%.255s", optarg);
			  break;
		  case 'i':
			  nInputs = atoi(optarg);
			  break;
		  case 't':
			  thresh = atof(optarg);
			  break;
		  case 'C':
			  nClasses = atoi(optarg);
			  break;
		  case '?':
			  usage();
			  exit(1);
		}
	}

	if (nClasses == 0 || nTest == 0 || nInputs == 0 || !strcmp(testFileName, "/dev/null") || !strcmp(dumpFileName, "/dev/null")) {
		usage();
		exit(1);
	}

	int			*testClasses = new int [nTest];
	double		**testData = new double * [nTest];
	for(i = 0; i < nTest; i++) testData[i] = new double [nInputs];

	loadData(testFileName, nTest, nInputs, testData, testClasses, NULL);

	WMSet::Initialize(0, 0, 0, 0, (WMSet::TWMType)modelType);

	if (nClasses == 2) {
		double	posSD = 0.0;
		double	negSD = 0.0;
		double	minPos = 0.0;
		double	maxPos = 0.0;
		double	minNeg = 0.0;
		double	maxNeg = 0.0;
		double	posAvg = 0.0;
		double	negAvg = 0.0;

		SD		*loadedSD = loadBinaryClassifier(dumpFileName);

		double	testAcc = loadedSD->getAccuracy(nTest, testData, testClasses, 1, thresh);
		fprintf(stderr, "\nOverall Accuracy = %7.2f%%\n", testAcc*100.0);
	}
	else {
		double	posSD = 0.0;
		double	negSD = 0.0;
		double	minPos = 0.0;
		double	maxPos = 0.0;
		double	minNeg = 0.0;
		double	maxNeg = 0.0;
		double	posAvg = 0.0;
		double	negAvg = 0.0;
		double	testAcc = 0.0;
		double	trainAcc = 0.0;
		int		**testClassArray = new int * [nClasses]; // not deleted
		double	*perClassAccuracy = new double [nClasses]; // not deleted
		SD		**sdArray = new SD * [nClasses]; // not deleted

		for(i = 0; i < nClasses; i++) {
			char	fname[512];
			SD		*sd = (SD *)0;
			int		*testC = new int [nTest];

			testClassArray[i] = testC;
			for(j = 0; j < nTest; j++) testC[j] = (testClasses[j] == i) ? 1 : 0;

			sprintf(fname, "%s.%d", dumpFileName, i);
			sdArray[i] = loadBinaryClassifier(fname);
			sd = sdArray[i];

			perClassAccuracy[i] = sd->getAccuracy(nTest, testData, testClassArray[i]);
		}
		testAcc = getMultiAccuracy(sdArray, testClasses, testData, nClasses, nTest, 1);

		fprintf(stderr, "\n");
		for(i = 0; i < nClasses; i++)
			fprintf(stderr, "Classifier %d Accuracy = %7.2f%%\n", i, 100.0*perClassAccuracy[i]);
		fprintf(stderr, "\nOverall Accuracy = %7.2f%%\n", 100.0*testAcc);
	}
	exit(0);
}
